from algebreb.util.numeros import entero_aleatorio

entero1 = entero_aleatorio(-100, 100)
print(entero1)

entero2 = entero_aleatorio(1, 10)
print(entero2)

entero3 = entero_aleatorio(-10, 10, 0)
print(entero3)

entero4 = entero_aleatorio(1, 3, 2)
print(entero4)


from algebreb.util.numeros_proxy import numeros_no_nulos

enteros1 = numeros_no_nulos('ZZ', 10, -100, 100)
print(enteros1)

enteros2 = numeros_no_nulos('ZZ', 10, -1, 1)
print(enteros2)

decimales1 = numeros_no_nulos('QQ', 10, -100, 100)
print(decimales1)

decimales2 = numeros_no_nulos('QQ', 10, -1, 1)
print(decimales2)

fracciones1 = numeros_no_nulos('QQ', 10, -100, 100, True)
print(fracciones1)

fracciones2 = numeros_no_nulos('QQ', 10, -1, 1, True)
print(fracciones2)


from algebreb.util.numeros_proxy import numero_no_nulo

entero1 = numero_no_nulo('ZZ', -100, 100)
print(entero1)

entero2 = numero_no_nulo('ZZ', -1000, 1000)
print(entero2)

decimal1 = numero_no_nulo('QQ', -100, 100)
print(entero1)

decimal2 = numero_no_nulo('QQ', -10, 10)
print(decimal2)

fraccion1 = numero_no_nulo('QQ', -100, 100, True)
print(fraccion1)

fraccion2 = numero_no_nulo('QQ', -10, 10, True)
print(fraccion2)


from sympy.solvers import solve
from sympy.abc import x
from sympy import *
from algebreb.expresiones.polinomios import Polinomio
from algebreb.ejercicios.tipos_ejercicios import DosOperandos

class EcuacionLineal(DosOperandos):
    def __init__(self, lado_derecho, lado_izquierdo):
        super(EcuacionLineal, self).__init__(lado_derecho, lado_izquierdo)
        self.oper = '='

    def operacion(self):
        ecuacion = self.op1 - self.op2
        variables = ecuacion.gens

        return solve(ecuacion.as_expr(), variables)

    def as_str(self):
        dict = {
            'operacion': self.oper,
            'operando1': str(self.op1.as_expr()),
            'operando2': str(self.op2.as_expr()),
            'enunciado': str(self.op1.as_expr()) + ' ' + self.oper + ' ' + str(self.op2.as_expr()),
            'solucion': [str(r) for r in self.res],
            'respuestas': self.respuestas
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper,
            'operando1': latex(self.op1.as_expr()),
            'operando2': latex(self.op2.as_expr()),
            'ecuacion': latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr()),
            'solucionArr': [latex(r) for r in self.res],
            'solucion': ','.join(str(r) for r in  self.res),
            'respuestas': self.respuestas,
            'enunciado': latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr()),
        }

        return dict


class EcuacionCuadratica(DosOperandos):
    def __init__(self, lado_derecho, lado_izquierdo) -> None:
        super(EcuacionCuadratica, self).__init__(lado_derecho, lado_izquierdo)
        self.oper = '='
        self.metodo = ''

    def operacion(self):
        ecuacion = self.op1 - self.op2
        variables = ecuacion.gens

        return solve(ecuacion.as_expr(), variables)

    def as_str(self):
        dict = {
            'operacion': self.oper,
            'operando1': str(self.op1.as_expr()),
            'operando2': str(self.op2.as_expr()),
            'enunciado': str(self.op1.as_expr()) + ' ' + self.oper + ' ' + str(self.op2.as_expr()),
            'solucion': [str(r) for r in self.res],
            'respuestas': self.respuestas
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper,
            'operando1': latex(self.op1.as_expr()),
            'operando2': latex(self.op2.as_expr()),
            'enunciado': latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr()),
            'solucionArr': [latex(r) for r in self.res],
            'solucion': ','.join(str(r) for r in  self.res),
            'respuestas': self.respuestas
        }

        return dict

poli = Polinomio(x**2-4,x)
poli2 = Polinomio(0, x)
ec = EcuacionCuadratica(poli, poli2)
print(ec.as_str())
print(ec.as_latex())


from algebreb.util.numeros_proxy import numeros

enteros1 = numeros('ZZ', 0, 20, -100, 100)
print(enteros1)

enteros2 = numeros('ZZ', 0, 20, 1, 100)
print(enteros2)

enteros3 = numeros('ZZ', 5, 5, 1, 100)
print(enteros3)

decimales1 = numeros('QQ', 0, 20, -10, 10)
print(decimales1)

decimales2 = numeros('QQ', 0, 20, 1, 100)
print(decimales2)

decimales3 = numeros('QQ', 5, 5, 1, 10)
print(decimales3)

fracciones1 = numeros('QQ', 0, 10, -10, 10, True)
print(fracciones1)

fracciones2 = numeros('QQ', 0, 10, 1, 10, True)
print(fracciones2)

fracciones3 = numeros('QQ', 5, 5, 1, 10, True)
print(fracciones3)


from algebreb.expresiones.polinomios import Polinomio, polinomio_raices_aleatorias, polinomio_coeficientes_aleatorios
from sympy.abc import x
from sympy import simplify

class FraccionAlgebraica:
    def __init__(self, num, den):
        self.num = Polinomio(num)
        self.den = Polinomio(den)

    def as_expr(self):
        return self.num.as_expr() / self.den.as_expr()

def fa_aleatoria(grado_num, grado_den, variables, dominio, coef_min, coef_max, fraccion=False, raices=True):
    if raices:
        p1 = polinomio_raices_aleatorias(grado_num, variables, dominio, coef_min, coef_max)
        p2 = polinomio_raices_aleatorias(grado_den, variables, dominio, coef_min, coef_max)
    else:
        p1 = polinomio_coeficientes_aleatorios(grado_num, variables, dominio, coef_min, coef_max)
        p2 = polinomio_coeficientes_aleatorios(grado_den, variables, dominio, coef_min, coef_max)

    fa = FraccionAlgebraica(p1, p2)

    return fa

def fa_propia_aleatoria(grado_num, variables, dominio, coef_min, coef_max, fraccion=False, raices=True):
    if raices:
        p1 = polinomio_raices_aleatorias(grado_num, variables, dominio, coef_min, coef_max)
        p2 = polinomio_raices_aleatorias(grado_num + 1, variables, dominio, coef_min, coef_max)
    else:
        p1 = polinomio_coeficientes_aleatorios(grado_num - 1, variables, dominio, coef_min, coef_max)
        p2 = polinomio_coeficientes_aleatorios(grado_num, variables, dominio, coef_min, coef_max)

    fa = FraccionAlgebraica(p1, p2)

    return fa

def fa_impropia_aleatoria(grado_den, variables, dominio, coef_min, coef_max, fraccion=False, raices=True):
    if raices:
        p1 = polinomio_raices_aleatorias(grado_den, variables, dominio, coef_min, coef_max)
        p2 = polinomio_raices_aleatorias(grado_den - 1, variables, dominio, coef_min, coef_max)
    else:
        p1 = polinomio_coeficientes_aleatorios(grado_den, variables, dominio, coef_min, coef_max)
        p2 = polinomio_coeficientes_aleatorios(grado_den - 1, variables, dominio, coef_min, coef_max)

    fa = FraccionAlgebraica(p1, p2)

    return fa


from sympy import *
from sympy.abc import a, b, c, x, y, z
from sympy import UnevaluatedExpr
from algebreb.expresiones.polinomios import Polinomio

inicio = '\\textrm{'
fin = '}'

def pasos_factor_comun_str(monomio, polinomio, producto):
    pasos = []

    monomio = monomio.as_expr()
    polinomio = polinomio.as_expr()
    producto = producto.as_expr()
    
    pasos.append('Se extrae el factor comun del polinomio {}, el cual es: {}'.format(producto, monomio))
    pasos.append('Se divide cada termino del polinomnio entre el factor comun, obteniendo {}'.format(polinomio))
    pasos.append('Se expresa el resultado de la siguiente manera {} = ({})({})'.format(producto, monomio, polinomio))

    return pasos

def pasos_factor_comun_latex(monomio, polinomio, producto):
    pasos = []

    monomio = monomio.as_expr()
    polinomio = polinomio.as_expr()
    producto = producto.as_expr()

    paso1 = inicio + 'Se extrae el factor comun del polinomio ' + fin + latex(producto) + inicio + ', el cual es: ' + fin + latex(monomio)
    paso2 = inicio + 'Se divide cada termino del polinomio entre el factor comun, obteniendo: ' + fin + latex(polinomio)
    paso3 = inicio + 'Se expresa el resultado de la siguiente manera: ' + fin + latex(producto) + inicio + ' = (' + fin + latex(monomio) + inicio + ')(' + fin + latex(polinomio) + inicio + ')' + fin
    
    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)

    return pasos

def pasos_diferencia_cuadrados_str(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    ta1 = monomios_bin1[0] * coeficientes_bin1[0]
    tb1 = monomios_bin1[1] * coeficientes_bin1[1]
    ta2 = monomios_bin2[0] * coeficientes_bin2[0]
    tb2 = monomios_bin2[1] * coeficientes_bin2[1]
    ta1p = monomios_prod[0] * coeficientes_prod[0]
    tb1p = monomios_prod[1] * coeficientes_prod[1] 

    pasos.append('Se extrae la raiz cuadrada del primer termino {} = {}'.format(ta1p, ta1))
    pasos.append('Se extrae la raiz cuadrada del segundo termino {} = {}'.format(-tb1p, tb1))
    pasos.append('Finalmente la factorizacion es: {} = ({})({})'.format(producto.as_expr(), bin1.as_expr(), bin2.as_expr()))

    return pasos

def pasos_diferencia_cuadrados_latex(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]
    
    ta1 = monomios_bin1[0] * coeficientes_bin1[0]
    tb1 = monomios_bin1[1] * coeficientes_bin1[1]
    ta2 = monomios_bin2[0] * coeficientes_bin2[0]
    tb2 = monomios_bin2[1] * coeficientes_bin2[1]
    ta1p = monomios_prod[0] * coeficientes_prod[0]
    tb1p = monomios_prod[1] * coeficientes_prod[1]

    paso1 = inicio + 'Se extrae la raiz cuadrada del primer termino ' + fin + latex(ta1p) + inicio + " = " + fin + latex(ta1)
    paso2 = inicio + 'Se extrae la raiz cuadrada del segundo termino' + fin + latex(-tb1p) + inicio + " = " + fin + latex(tb1)
    paso3 = inicio + 'Finalmente la factorizacion es: ' + fin + latex(producto.as_expr()) + inicio + ' = (' + fin + latex(bin1.as_expr()) + inicio + ')(' + fin + latex(bin2.as_expr()) + inicio + ')' + fin
    
    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)

    return pasos

def pasos_trinomio_cuadrado_perfecto_str(binomio, producto):
    pasos = []

    coeficientes_binomio = binomio.coeffs()
    monomios_binomio = [prod(x**k for x, k in zip(binomio.gens, mon)) for mon in binomio.monoms()]
    
    coeficientes_producto = producto.coeffs()
    monomios_producto = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    ta1 = monomios_binomio[0] * coeficientes_binomio[0]
    tb1 = monomios_binomio[1] * coeficientes_binomio[1]

    ta1p = monomios_producto[0] * coeficientes_producto[0]
    tb1p = monomios_producto[1] * coeficientes_producto[1]
    tc1p = monomios_producto[2] * coeficientes_producto[2]

    signo = ''

    if coeficientes_producto[1] > 0:
        signo = '+'
    else:
        signo = '-'

    bin_expr = binomio.as_expr()

    with evaluate(False):
        u_factor = UnevaluatedExpr(bin_expr**2)

    pasos.append('Se extrae la raíz cuadrada del primer termino {}: {}'.format(ta1p, ta1))
    pasos.append('Se extrae la raiz cuadrada del ultimo termino {}: {}'.format(tc1p, tb1))
    pasos.append('Se extrae el signo del termino de en medio {}, que en este caso es {}'.format(tb1p, signo))
    pasos.append('Se unen las dos raices cuadradas mediante el signo anterior y el resultado se expresas asi: {} = {}'.format(producto.as_expr(), u_factor))

    return pasos

def pasos_trinomio_cuadrado_perfecto_latex(binomio, producto):
    pasos = []

    coeficientes_binomio = binomio.coeffs()
    monomios_binomio = [prod(x**k for x, k in zip(binomio.gens, mon)) for mon in binomio.monoms()]
    
    coeficientes_producto = producto.coeffs()
    monomios_producto = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    ta1 = monomios_binomio[0] * coeficientes_binomio[0]
    tb1 = monomios_binomio[1] * coeficientes_binomio[1]

    ta1p = monomios_producto[0] * coeficientes_producto[0]
    tb1p = monomios_producto[1] * coeficientes_producto[1]
    tc1p = monomios_producto[2] * coeficientes_producto[2]

    signo = ''

    if coeficientes_producto[1] > 0:
        signo = '+'
    else:
        signo = '-'

    bin_expr = binomio.as_expr()

    with evaluate(False):
        u_factor = UnevaluatedExpr(bin_expr**2)

    paso1 = inicio + 'Se extrae la raíz cuadrada del primer termino ' + fin + latex(ta1p) + inicio + ': ' + fin + latex(ta1)
    paso2 = inicio + 'Se extrae la raíz cuadrada del ultimo termino ' + fin + latex(tc1p) + inicio + ': ' + fin + latex(tb1)
    paso3 = inicio + 'Se extrae el signo del termino de en medio ' + fin + latex(tb1p) + inicio + ', que en este caso es ' + fin + latex(signo)
    paso4 = inicio + 'Se unen las dos raices cuadradas mediante el signo anterior y el resultado se expresas asi: ' + fin + latex(producto.as_expr(), long_frac_ratio=oo) + inicio + ' = ' + fin + latex(u_factor, long_frac_ratio=oo)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)

    return pasos

def pasos_cubo_perfecto_binomios_str(binomio, producto):
    pasos = []

    coeficientes_binomio = binomio.coeffs()
    monomios_binomio = [prod(x**k for x, k in zip(binomio.gens, mon)) for mon in binomio.monoms()]
    
    coeficientes_producto = producto.coeffs()
    monomios_producto = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    ta1 = monomios_binomio[0] * coeficientes_binomio[0]
    tb1 = monomios_binomio[1] * coeficientes_binomio[1]

    ta1p = monomios_producto[0] * coeficientes_producto[0]
    tb1p = monomios_producto[1] * coeficientes_producto[1]
    tc1p = monomios_producto[2] * coeficientes_producto[2]
    td1p = monomios_producto[2] * coeficientes_producto[2]

    bin_expr = binomio.as_expr()

    with evaluate(False):
        u_factor = UnevaluatedExpr(bin_expr**3)

    pasos.append('Se extrae la raíz cubica del primer termino {}: {}'.format(ta1p, ta1))
    pasos.append('Se extrae la raiz cubica del ultimo termino {}: {}'.format(td1p, tb1))
    pasos.append('Si el signo del segundo y cuarto termino es + y - o viceversa, se escoge el signo -, si ambos signos son +, se escoge +')
    pasos.append('Se unen las dos raices cubicas mediante el signo que se escogió y el resultado se expresas asi: {} = {}'.format(producto.as_expr(), u_factor))

    return pasos

def pasos_cubo_perfecto_binomios_latex(binomio, producto):
    pasos = []

    coeficientes_binomio = binomio.coeffs()
    monomios_binomio = [prod(x**k for x, k in zip(binomio.gens, mon)) for mon in binomio.monoms()]
    
    coeficientes_producto = producto.coeffs()
    monomios_producto = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    ta1 = monomios_binomio[0] * coeficientes_binomio[0]
    tb1 = monomios_binomio[1] * coeficientes_binomio[1]

    ta1p = monomios_producto[0] * coeficientes_producto[0]
    tb1p = monomios_producto[1] * coeficientes_producto[1]
    tc1p = monomios_producto[2] * coeficientes_producto[2]
    td1p = monomios_producto[2] * coeficientes_producto[2]

    bin_expr = binomio.as_expr()

    with evaluate(False):
        u_factor = UnevaluatedExpr(bin_expr**3)

    paso1 = inicio + 'Se extrae la raiz cubica del primer termino ' + fin + latex(ta1p) + inicio + ': ' + fin + latex(ta1)
    paso2 = inicio + 'Se extrae la raiz cubica del ultimo termino ' + fin + latex(td1p) + inicio + ': ' + fin + latex(tb1)
    paso3 = inicio + 'Si el signo del segundo y cuarto termino es + y - o viceversa, se escoge el signo -, si ambos signos son +, se escoge +' + fin
    paso4 = inicio + 'Se unen las dos raices cubicas mediante el signo que se escogio y el resultado se expresas asi: ' + fin + latex(producto.as_expr()) + inicio + ' = ' + fin + latex(u_factor)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)

    return pasos

def pasos_trinomio_forma1_str(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    bin1t1 = coeficientes_bin1[0] * monomios_bin1[0]
    bin1t2 = coeficientes_bin1[1] * monomios_bin1[1]

    bin2t1 = coeficientes_bin2[0] * monomios_bin2[0]
    bin2t2 = coeficientes_bin2[1] * monomios_bin2[1]

    prodt1 = coeficientes_prod[0] * monomios_prod[0]
    prodt2 = coeficientes_prod[1] * monomios_prod[1]
    prodt3 = coeficientes_prod[2] * monomios_prod[2]

    pasos.append('Se extrae la raiz cuadrada del termino cuadratico y se coloca el resultado en ambos factores ({}  )({}  )'.format(bin1t1, bin2t1))
    pasos.append('Se buscan dos numeros que sumados den el coeficiente del termino {} y multiplicados den el termino {}, los cuales son {} y {}'.format(prodt2, prodt3,coeficientes_bin1[1], coeficientes_bin2[1]))
    pasos.append('Se colocan los numeros dentro de los factores del paso de esta manera: ({})({})'.format(bin1.as_expr(), bin2.as_expr()))

    return pasos

def pasos_trinomio_forma1_latex(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    bin1t1 = coeficientes_bin1[0] * monomios_bin1[0]
    bin1t2 = coeficientes_bin1[1] * monomios_bin1[1]

    bin2t1 = coeficientes_bin2[0] * monomios_bin2[0]
    bin2t2 = coeficientes_bin2[1] * monomios_bin2[1]

    prodt1 = coeficientes_prod[0] * monomios_prod[0]
    prodt2 = coeficientes_prod[1] * monomios_prod[1]
    prodt3 = coeficientes_prod[2] * monomios_prod[2]

    paso1 = inicio + 'Se extrae la raiz cuadrada del termino cuadratico y se coloca el resultado en ambos factores (' + fin + latex(bin1t1) + inicio + '  )(' + fin + latex(bin2t1) + inicio + '  )' + fin
    paso2 = inicio + 'Se buscan dos numeros que sumados den el coeficiente del termino ' + fin + latex(prodt2) + inicio + ' y multiplicados den el coeficiente del termino ' + fin + latex(prodt3) + inicio + ', los cuales son ' + fin + str(coeficientes_bin1[1]) + inicio + ' y ' + fin +  str(coeficientes_bin2[1])
    paso3 = inicio + 'Se colocan los numeros dentro de los factores del paso de esta manera: (' + fin + latex(bin1.as_expr()) + inicio + ')(' + fin + latex(bin2.as_expr()) + inicio + ')' + fin

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)

    return pasos

def pasos_trinomio_forma2_str(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    bin1t1 = coeficientes_bin1[0] * monomios_bin1[0]
    bin1t2 = coeficientes_bin1[1] * monomios_bin1[1]

    bin2t1 = coeficientes_bin2[0] * monomios_bin2[0]
    bin2t2 = coeficientes_bin2[1] * monomios_bin2[1]

    prodt1 = coeficientes_prod[0] * monomios_prod[0]
    prodt2 = coeficientes_prod[1] * monomios_prod[1]
    prodt3 = coeficientes_prod[2] * monomios_prod[2]

    pasos.append('Se buscan dos numeros que multiplicados den el coeficiente del termino {}, los cuales son {} y {}'.format(prodt1, coeficientes_bin1[0], coeficientes_bin2[0]))
    pasos.append('Se buscan dos numeros que multiplicados den el termino {}, los cuales son {} y {}'.format(prodt3, coeficientes_bin1[1], coeficientes_bin2[1]))
    pasos.append('Se multiplican ({})({}) = {} y ({})({}) = {}'.format(coeficientes_bin1[0], coeficientes_bin2[1], coeficientes_bin1[0]*coeficientes_bin2[1], coeficientes_bin1[1], coeficientes_bin2[0], coeficientes_bin1[1]*coeficientes_bin2[0]))
    pasos.append('Se suman los productos anteriores ({}) + ({}) = {}'.format(coeficientes_bin1[0]*coeficientes_bin2[1], coeficientes_bin1[1]*coeficientes_bin2[0], coeficientes_bin1[0]*coeficientes_bin2[1] + coeficientes_bin1[1]*coeficientes_bin2[0]))
    pasos.append('Se acomodan los productos obtenidos de la siguiente manera: ({})({}) = {}'.format(bin1.as_expr(), bin2.as_expr(), producto.as_expr()))

    return pasos

def pasos_trinomio_forma2_latex(bin1, bin2, producto):
    pasos = []

    coeficientes_bin1 = bin1.coeffs()
    monomios_bin1 = [prod(x**k for x, k in zip(bin1.gens, mon)) for mon in bin1.monoms()]

    coeficientes_bin2 = bin2.coeffs()
    monomios_bin2 = [prod(x**k for x, k in zip(bin2.gens, mon)) for mon in bin2.monoms()]

    coeficientes_prod = producto.coeffs()
    monomios_prod = [prod(x**k for x, k in zip(producto.gens, mon)) for mon in producto.monoms()]

    bin1t1 = coeficientes_bin1[0] * monomios_bin1[0]
    bin1t2 = coeficientes_bin1[1] * monomios_bin1[1]

    bin2t1 = coeficientes_bin2[0] * monomios_bin2[0]
    bin2t2 = coeficientes_bin2[1] * monomios_bin2[1]

    prodt1 = coeficientes_prod[0] * monomios_prod[0]
    prodt2 = coeficientes_prod[1] * monomios_prod[1]
    prodt3 = coeficientes_prod[2] * monomios_prod[2]

    p1 = coeficientes_bin1[0]*coeficientes_bin2[1]
    p2 = coeficientes_bin1[1]*coeficientes_bin2[0]

    paso1 = inicio + 'Se buscan dos numeros que multiplicados den el coeficiente del termino ' + fin + latex(prodt1) + inicio + ', los cuales son ' + fin + str(coeficientes_bin1[0]) + inicio + ' y ' + fin + str(coeficientes_bin2[0])
    paso2 = inicio + 'Se buscan dos numeros que multiplicados den el termino ' + fin + latex(prodt3) + inicio + ', los cuales son ' + fin + str(coeficientes_bin1[1]) + inicio + ' y ' + fin + str(coeficientes_bin2[1])
    paso3 = inicio + 'Se multiplican (' + fin + str(coeficientes_bin1[0]) + inicio + ')(' + fin + str(coeficientes_bin2[1]) + inicio + ') = ' + fin + str(coeficientes_bin1[0]*coeficientes_bin2[1]) + inicio + ' y (' + fin + str(coeficientes_bin1[1]) + inicio + ')(' + fin + str(coeficientes_bin2[0]) + inicio + ') = ' + fin + str(coeficientes_bin1[1]*coeficientes_bin2[0])
    paso4 = inicio + 'Se suman los productos anteriores (' + fin + str(coeficientes_bin1[0]*coeficientes_bin2[1]) + inicio + ') + (' + fin + str(coeficientes_bin1[1]*coeficientes_bin2[0]) + inicio + ') = ' + fin + str(coeficientes_bin1[0]*coeficientes_bin2[1] + coeficientes_bin1[1]*coeficientes_bin2[0])
    paso5 = inicio + 'Se acomodan los productos obtenidos de la siguiente manera: (' + fin + latex(bin1.as_expr()) + inicio + ')(' + fin + latex(bin2.as_expr()) + inicio + ') = ' + fin + latex(producto.as_expr())

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)
    pasos.append(paso5)
    
    return pasos


from sympy.abc import x, y, z
from algebreb.expresiones.polinomios import polinomio_coeficientes_aleatorios

p1 = polinomio_coeficientes_aleatorios(2, [x, y], 'ZZ', 1, 14)
print(p1)
print(p1.total_degree())

p2 = polinomio_coeficientes_aleatorios(5, [x], 'ZZ', -10, 10)
print(p2)
print(p2.total_degree())

p3 = polinomio_coeficientes_aleatorios(3,[x, y], 'ZZ', -67, 54, False, False)
print(p3)

p4 = polinomio_coeficientes_aleatorios(2, [x, y, z], 'QQ', -13, 27)
print(p4)

p5 = polinomio_coeficientes_aleatorios(7, [y, z], 'QQ', -1000, 1000, False, False)
print(p5)

p6 = polinomio_coeficientes_aleatorios(3, [x, z], 'QQ', 1, 20, True, True)
print(p6)




from sympy import *
from sympy.abc import a, b, c, x, y, z
from sympy import UnevaluatedExpr
from algebreb.expresiones.polinomios import Polinomio
from algebreb.expresiones.fracciones_algebraicas import FraccionAlgebraica

inicio = '\\textrm{'
fin = '}'

def pasos_suma_fracciones_algebraicas_str(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        ad = f1n_factor * f2d_factor
        bc = f1d_factor * f2n_factor
        nuevo_num = ad + bc
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f1))
    pasos.append('Se factoriza el numerador y denominador de la segunda fraccion: {}'.format(u_f2))
    pasos.append('Se multiplica el numerador de la primer fraccion por el denominador de la segunda: {}'.format(ad))
    pasos.append('Se multiplica el denominador de la primer fraccion por el numerador de la segunda: {}'.format(bc))
    pasos.append('Se suman los dos productos anteriores para obtener el nuevo numerador: {}'.format(nuevo_num))
    pasos.append('Se multiplican los 2 denominadores de las fracciones para obtener el nuevo denominador: {}'.format(nuevo_den))
    pasos.append('Se acomoda el nuevo numerador y denominador: {}'.format(n_frac))
    pasos.append('Se hacen los productos y sumas correspondientes. Finalmente: {}'.format(n_frac_factor))

    return pasos

def pasos_suma_fracciones_algebraicas_latex(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        ad = f1n_factor * f2d_factor
        bc = f1d_factor * f2n_factor
        nuevo_num = ad + bc
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    paso1 = inicio + 'Se factoriza el numerador y denominador de la primera fraccion: ' + fin + latex(u_f1)
    paso2 = inicio + 'Se factoriza el numerador y denominador de la segunda fraccion: ' + fin + latex(u_f2)
    paso3 = inicio + 'Se multiplica el numerador de la primer fraccion por el denominador de la segunda: ' + fin + latex(ad)
    paso4 = inicio + 'Se multiplica el denominador de la primer fraccion por el numerador de la segunda: ' + fin + latex(bc)
    paso5 = inicio + 'Se suman los dos productos anteriores para obtener el nuevo numerador: ' + fin + latex(nuevo_num)
    paso6 = inicio + 'Se multiplican los 2 denominadores de las fracciones para obtener el nuevo denominador: ' + fin + latex(nuevo_den)
    paso7 = inicio + 'Se acomoda el nuevo numerador y denominador: ' + fin + latex(n_frac)
    paso8 = inicio + 'Se hacen los productos y sumas correspondientes. Finalmente: ' + fin + latex(n_frac_factor)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)
    pasos.append(paso5)
    pasos.append(paso6)
    pasos.append(paso7)
    pasos.append(paso8)

    return pasos

def pasos_resta_fracciones_algebraicas_str(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        ad = f1n_factor * f2d_factor
        bc = f1d_factor * f2n_factor
        nuevo_num = ad - bc
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f1))
    pasos.append('Se factoriza el numerador y denominador de la segunda fraccion: {}'.format(u_f2))
    pasos.append('Se multiplica el numerador de la primer fraccion por el denominador de la segunda: {}'.format(ad))
    pasos.append('Se multiplica el denominador de la primer fraccion por el numerador de la segunda: {}'.format(bc))
    pasos.append('Se restan los dos productos anteriores para obtener el nuevo numerador: {}'.format(nuevo_num))
    pasos.append('Se multiplican los 2 denominadores de las fracciones para obtener el nuevo denominador: {}'.format(nuevo_den))
    pasos.append('Se acomoda el nuevo numerador y denominador: {}'.format(n_frac))
    pasos.append('Se hacen los productos y restas correspondientes. Finalmente: {}'.format(n_frac_factor))

    return pasos

def pasos_resta_fracciones_algebraicas_latex(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        ad = f1n_factor * f2d_factor
        bc = f1d_factor * f2n_factor
        nuevo_num = ad - bc
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    paso1 = inicio + 'Se factoriza el numerador y denominador de la primera fraccion: ' + fin + latex(u_f1)
    paso2 = inicio + 'Se factoriza el numerador y denominador de la segunda fraccion: ' + fin + latex(u_f2)
    paso3 = inicio + 'Se multiplica el numerador de la primer fraccion por el denominador de la segunda: ' + fin + latex(ad)
    paso4 = inicio + 'Se multiplica el denominador de la primer fraccion por el numerador de la segunda: ' + fin + latex(bc)
    paso5 = inicio + 'Se restan los dos productos anteriores para obtener el nuevo numerador: ' + fin + latex(nuevo_num)
    paso6 = inicio + 'Se multiplican los 2 denominadores de las fracciones para obtener el nuevo denominador: ' + fin + latex(nuevo_den)
    paso7 = inicio + 'Se acomoda el nuevo numerador y denominador: ' + fin + latex(n_frac)
    paso8 = inicio + 'Se hacen los productos y sumas correspondientes. Finalmente: ' + fin + latex(n_frac_factor)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)
    pasos.append(paso5)
    pasos.append(paso6)
    pasos.append(paso7)
    pasos.append(paso8)

    return pasos

def pasos_mult_fracciones_algebraicas_str(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        nuevo_num = f1n_factor * f2n_factor
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f1))
    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f2))
    pasos.append('Se multiplican los numeradores de las fracciones: {}'.format(nuevo_num))
    pasos.append('Se multiplican los denominadores de las fracciones: {}'.format(nuevo_den))
    pasos.append('Se acomoda el nuevo numerador y denominador: {}'.format(n_frac))
    pasos.append('Se cancelan terminos. Finalmente: {}'.format(n_frac_factor))

    return pasos

def pasos_mult_fracciones_algebraicas_latex(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2d_factor
        nuevo_num = f1n_factor * f2n_factor
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    paso1 = inicio + 'Se factoriza el numerador y denominador de la primera fraccion: ' + fin + latex(u_f1)
    paso2 = inicio + 'Se factoriza el numerador y denominador de la segunda fraccion: ' + fin + latex(u_f2)
    paso3 = inicio + 'Se multiplican los numeradores de las fracciones: ' + fin + latex(nuevo_num)
    paso4 = inicio + 'Se multiplican los denominadores de las fracciones: ' + fin + latex(nuevo_den)
    paso5 = inicio + 'Se acomoda el nuevo numerador y denominador: ' + fin + latex(n_frac)
    paso6 = inicio + 'Se cancelan terminos. Finalmente: : ' + fin + latex(n_frac_factor)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)
    pasos.append(paso5)
    pasos.append(paso6)

    return pasos

def pasos_div_fracciones_algebraicas_str(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2n_factor
        nuevo_num = f1n_factor * f2d_factor
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f1))
    pasos.append('Se factoriza el numerador y denominador de la primera fraccion: {}'.format(u_f2))
    pasos.append('Se multiplican el numerador de la primer fraccion por el denominador de la segunda: {}'.format(nuevo_num))
    pasos.append('Se multiplican el denominador de la primer fraccion por el numerador de la segunda: {}'.format(nuevo_den))
    pasos.append('Se acomoda el nuevo numerador y denominador: {}'.format(n_frac))
    pasos.append('Se cancelan terminos. Finalmente: {}'.format(n_frac_factor))

def pasos_div_fracciones_algebraicas_latex(f1, f2):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()
    f2n = f2.num.as_expr()
    f2d = f2.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)
    f2n_factor = factor(f2n)
    f2d_factor = factor(f2d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor
        u_f2 = f2n_factor / f2d_factor
        nuevo_den = f1d_factor * f2n_factor
        nuevo_num = f1n_factor * f2d_factor
        n_frac = nuevo_num / nuevo_den

    n_frac_factor = simplify(n_frac)

    paso1 = inicio + 'Se factoriza el numerador y denominador de la primera fraccion: ' + fin + latex(u_f1)
    paso2 = inicio + 'Se factoriza el numerador y denominador de la segunda fraccion: ' + fin + latex(u_f2)
    paso3 = inicio + 'Se multiplican el numerador de la primer fraccion por el denominador de la segunda: ' + fin + latex(nuevo_num)
    paso4 = inicio + 'Se multiplican el denominador de la primer fraccion por el numerador de la segunda: ' + fin + latex(nuevo_den)
    paso5 = inicio + 'Se acomoda el nuevo numerador y denominador: ' + fin + latex(n_frac)
    paso6 = inicio + 'Se cancelan terminos. Finalmente: ' + fin + latex(n_frac_factor)

    pasos.append(paso1)
    pasos.append(paso2)
    pasos.append(paso3)
    pasos.append(paso4)
    pasos.append(paso5)
    pasos.append(paso6)

    return pasos

def pasos_simp_fracciones_algebraicas_str(f1):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor

    n_frac_factor = simplify(u_f1)

    pasos.append('Se factoriza el numerador y denominador de la fraccion: {}'.format(u_f1))
    pasos.append('Se cancelan terminos: {}'.format(n_frac_factor))

    return pasos

def pasos_simp_fracciones_algebraicas_latex(f1):
    pasos = []
    
    f1n = f1.num.as_expr()
    f1d = f1.den.as_expr()

    f1n_factor = factor(f1n)
    f1d_factor = factor(f1d)

    with evaluate(False):
        u_f1 = f1n_factor / f1d_factor

    n_frac_factor = simplify(u_f1)

    paso1 = inicio + 'Se factoriza el numerador y denominador de la fraccion: ' + fin + latex(u_f1)
    paso2 = inicio + 'Se cancelan terminos: ' + fin + latex(n_frac_factor)

    pasos.append(paso1)
    pasos.append(paso2)

    return pasos

p1 = Polinomio(x+2, x)
p2 = Polinomio(x**2+5*x+6, x)
p3 = Polinomio(x+2, x)
p4 = Polinomio(x**2+4*x+3, x)

f1 = FraccionAlgebraica(p1, p2)
f2 = FraccionAlgebraica(p3, p4)

f_pasos = pasos_simp_fracciones_algebraicas_latex(f1)






from random import randint, uniform
from fractions import Fraction

def nulos(cantidad):
    return [0] * cantidad

def entero_aleatorio(min, max, excepcion=None):
    entero = randint(min, max)

    if excepcion != None:
        while entero == excepcion:
            entero = randint(min, max)

        return entero
    else:
        return entero
    
def enteros_aleatorios(min, max, cantidad, excepcion=None):
    enteros = []

    for entero in range(cantidad):
        if excepcion != None:
            entero = entero_aleatorio(min, max, excepcion)
        else:
            entero = entero_aleatorio(min, max)

        enteros.append(entero)

    return enteros

def decimal_aleatorio(min, max, excepcion=None):
    decimal = uniform(min, max)

    if excepcion != None:
        while decimal == excepcion:
            decimal = randint(min, max)

        return round(decimal, 2)
    else:
        return round(decimal, 2)

def decimales_aleatorios(min, max, cantidad, excepcion=None):
    decimales = []

    for decimal in range(cantidad):
        decimal = decimal_aleatorio(min, max, excepcion)
        decimales.append(round(decimal, 2))

    return decimales

def fraccion_aleatoria(min, max, excepcion=None):
    numerador = entero_aleatorio(min, max, 0) # No queremos fracciones nulas
    denominador = entero_aleatorio(min, max, 0) # El denominador no puede ser cero

    fraccion = Fraction(numerador, denominador)

    if excepcion != None:
        while fraccion == excepcion:
            fraccion = fraccion_aleatoria(min, max)

        return fraccion
    else:
        return fraccion

def fracciones_aleatorias(min, max, cantidad, excepcion=None):
    fracciones = []

    for fraccion in range(cantidad):
        fraccion = fraccion_aleatoria(min, max, excepcion)
        fracciones.append(fraccion)

    return fracciones


from sympy.abc import a, b, c,x, y, z
from algebreb.util.terminos import generar_terminos_grado

terms1 = generar_terminos_grado([x, y], 2)
print(terms1)

terms2 = generar_terminos_grado([x, y], 1)
print(terms2)

terms3 = generar_terminos_grado([x, y], 3)
print(terms3)

terms4 = generar_terminos_grado([x, y, z], 2)
print(terms4)

terms5 = generar_terminos_grado([a, b, c, x, y, z], 2)
print(terms5)

terms6 = generar_terminos_grado([a], 2)
print(terms6)

terms7 = generar_terminos_grado([b, c], 7)
print(terms7)


from sympy import *
from sympy.abc import x, y
from algebreb.ejercicios.tipos_ejercicios import UnOperando, DosOperandos
from algebreb.expresiones.fracciones_algebraicas import fa_propia_aleatoria
from algebreb.pasos.fracciones_algebraicas import (pasos_mult_fracciones_algebraicas_latex, 
                                                   pasos_resta_fracciones_algebraicas_latex, 
                                                   pasos_suma_fracciones_algebraicas_latex,
                                                   pasos_div_fracciones_algebraicas_latex, 
                                                   pasos_simp_fracciones_algebraicas_latex)

class SumaFracAlg(DosOperandos):
    def __init__(self, op1, op2):
        super(SumaFracAlg, self).__init__(op1, op2)
        self.oper = '+'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())
    
    def operacion(self):
        return simplify(self.op1.as_expr() + self.op2.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_suma_fracciones_algebraicas_latex(self.op1, self.op2)

class RestaFracAlg(DosOperandos):
    def __init__(self, op1, op2):
        super().__init__(op1, op2)
        self.oper = '-'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())
    
    def operacion(self):
        return simplify(self.op1.as_expr() - self.op2.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_resta_fracciones_algebraicas_latex(self.op1, self.op2)

class MultFracAlg(DosOperandos):
    def __init__(self, op1, op2):
        super().__init__(op1, op2)
        self.oper = '*'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())
    
    def operacion(self):
        return simplify(self.op1.as_expr() * self.op2.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_mult_fracciones_algebraicas_latex(self.op1, self.op2)

class DivFracAlg(DosOperandos):
    def __init__(self, op1, op2):
        super().__init__(op1, op2)
        self.oper = '/'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())
    
    def operacion(self):
        return simplify(self.op1.as_expr() / self.op2.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_div_fracciones_algebraicas_latex(self.op1, self.op2)

class SimpFracAlg(UnOperando):
    def __init__(self, op1):
        super().__init__(op1)
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return simplify(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_simp_fracciones_algebraicas_latex(self.op1)



from algebreb.expresiones.polinomios import (polinomio_coeficientes_aleatorios,
                                            polinomio_homogeneo_aleatorio,
                                            trinomio_aleatorio,
                                            binomio_aleatorio,
                                            monomio_aleatorio)
from algebreb.expresiones.fracciones_algebraicas import (fa_aleatoria as fa,
                                                        fa_propia_aleatoria,
                                                        fa_impropia_aleatoria)

def polinomio_aleatorio(tipo, grado, variabs, dom, cmin, cmax, fracc=False, compl=True):
    if tipo == 'M':
        return monomio_aleatorio(grado, variabs, dom, cmin, cmax, fracc)
    elif tipo == 'B':
        return binomio_aleatorio(grado, variabs, dom, cmin, cmax, fracc)
    elif tipo == 'T':
        return trinomio_aleatorio(grado, variabs, dom, cmin, cmax, fracc)
    elif tipo == 'H':
        return polinomio_homogeneo_aleatorio(grado, variabs, dom, cmin, cmax, fracc, compl)
    else:
        return polinomio_coeficientes_aleatorios(grado, variabs, dom, cmin, cmax, fracc, compl)

def fa_aleatoria(tipo, variabs, dom, crmin, crmax, fracc, raices, gnum=1, gden=1):
    if tipo == 'P':
        return fa_propia_aleatoria(gden, variabs, dom, crmin, crmax, fracc, raices)
    elif tipo == 'I':
        return fa_impropia_aleatoria(gnum, variabs, dom, crmin, crmax, fracc, raices)
    else:
        return fa(gnum, gden, variabs, dom, crmin, crmax, fracc, raices)


class Lista:
    def __init__(self, caracteristicas):
        self.caracteristicas = caracteristicas
        self.ejercicios = self.lista_ejercicios()
        self.instrucciones = ''

    def lista_ejercicios(self):
        return []

    def as_str_latex(self):
        dicc = {}

        lista_latex = []
        lista_str = []

        for ejercicio in self.ejercicios:
            lista_latex.append(ejercicio.as_latex())
            lista_str.append(ejercicio.as_str())

        dicc['instrucciones'] = self.instrucciones
        dicc['latex'] = lista_latex
        dicc['str'] = lista_str

        return dicc

"""
    def as_str_latex(self):
        dicc = {}

        nombre = "ejercicio"
        numero = 1

        for ejercicio in self.ejercicios:
            clave = nombre + str(numero)

            dicc[clave] = ejercicio.as_str_latex()

            numero += 1
            
        return dicc
"""



from sympy import *

from algebreb.expresiones.polinomios import Polinomio

class UnOperando():
    def __init__(self, op1):
        self.oper = ''
        self.op1 = op1
        self.res = self.operacion()
        self.respuestas = []
        self.pasos = {}
        self.enunciado = ''

    def set_op1(self, op1):
        self.op1 = op1

    def get_op1(self):
        return self.op1

    def operacion(self):
        return

    def ver_pasos(self):
        return

    def as_str(self):
        dict = {
            'operacion': str(self.oper),
            'operando1': str(self.op1.as_expr()),
            'solucion': str(self.res.as_expr()),
            'respuestas': [str(r.as_expr()) for r in self.respuestas]
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper, 
            'operando1': latex(self.op1.as_expr()),
            'solucion': latex(self.res.as_expr()),
            'enunciado': self.enunciado,
            'respuestas': [latex(r.as_expr()) for r in self.respuestas],
            'pasos': self.pasos
        }

        return dict

    def as_str_latex(self):
        diccionario = {}

        diccionario['string'] = self.as_str()
        diccionario['latex'] = self.as_latex()

        return diccionario


class DosOperandos():
    def __init__(self, op1, op2):
        self.oper = ''
        self.op1 = op1
        self.op2 = op2
        self.res = self.operacion()
        self.respuestas = []
        self.pasos = {}
        self.enunciado = ''

    def set_op1(self, op1):
        self.op1 = op1

    def get_op1(self):
        return self.op1

    def set_op2(self, op2):
        self.op2 = op2

    def get_op2(self):
        return self.op2

    def operacion(self):
        return [r.as_expr() for r in self.respuestas] #if self.respuestas == [] else []

    def ver_pasos(self):
        return
    
    def respuestas_str(self):
        return

    def as_str(self):
        dict = {
            'operacion': str(self.oper),
            'operando1': str(self.op1.as_expr()),
            'operando2': str(self.op2.as_expr()),
            'solucion': str(self.res.as_expr()),
            'respuestas': [str(r.as_expr()) for r in self.respuestas]
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper,
            'operando1': latex(self.op1.as_expr()),
            'operando2': latex(self.op2.as_expr()),
            'enunciado': self.enunciado,
            'solucion': latex(self.res.as_expr()),
            'respuestas': [latex(r.as_expr()) for r in self.respuestas],
            'pasos': self.pasos
        }

        return dict

    def as_str_latex(self):
        diccionario = {}

        diccionario['string'] = self.as_str()
        diccionario['latex'] = self.as_latex()

        return diccionario


import fractions
from math import lcm

from algebreb.util.numeros_proxy import numeros

def vieta(raices, grado):
    coeficientes = [0] * (grado + 1)

    coeficientes[grado] = 1
    for i in range(1, grado + 1):
        for j in range(grado - i - 1, grado):
            coeficientes[j] += ((-1) * raices[i - 1] * coeficientes[j + 1])

    coeficientes = coeficientes[::-1]

    return coeficientes

from random import choice

def suma_azar(numero):
    if numero == 1:
        return 0, 1
        
    techo = numero
    piso = 0
    lista = []

    while piso < numero - 1:
        piso += 1
        techo -= 1

        tupla = (piso, techo)
        lista.append(tupla)

    
    t = choice(lista)

    s1 = t[0]
    s2 = t[1]

    return s1, s2

def mcm_fracciones(fracciones):
    denominadores = []

    for f in fracciones:
        den = f.denominator
        denominadores.append(den)

    print(denominadores)
    mcm = lcm(*denominadores)

    return abs(mcm)


from sympy.abc import x, y
from sympy import *
from algebreb.ejercicios.tipos_ejercicios import UnOperando, DosOperandos
from algebreb.expresiones.polinomios import binomio_aleatorio
from algebreb.pasos.productos_notables import (pasos_binomio_al_cuadrado_latex,
                                               pasos_binomio_al_cubo_latex,
                                               pasos_binomios_conjugados_latex, 
                                               pasos_binomios_forma1_latex, pasos_binomios_forma2_latex, pasos_trinomio_al_cuadrado_latex)

class BinomioAlCuadrado(UnOperando):
    def __init__(self, op1):
        super(BinomioAlCuadrado, self).__init__(op1)
        self.oper = ''
        self.enunciado = latex(self.op1.as_expr()**2)

    def operacion(self):
        return self.op1**2

    def ver_pasos(self):   
        self.pasos = pasos_binomio_al_cuadrado_latex(self.op1)

class BinomioAlCubo(UnOperando):
    def __init__(self, op1):
        super(BinomioAlCubo, self).__init__(op1)
        self.oper = ''
        self.enunciado = latex(self.op1.as_expr()**3)

    def operacion(self):
        return self.op1**3

    def ver_pasos(self):   
        self.pasos = pasos_binomio_al_cubo_latex(self.op1)

class BinomiosConjugados(DosOperandos):
    def __init__(self, op1, op2):
        super(BinomiosConjugados, self).__init__(op1, op2)
        self.oper = '*'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())

    def operacion(self):
        return self.op1 * self.op2

    def ver_pasos(self):   
        self.pasos = pasos_binomios_conjugados_latex(self.op1, self.op2)

class BinomiosFormaI(DosOperandos):
    def __init__(self, op1, op2):
        super(BinomiosFormaI, self).__init__(op1, op2)
        self.oper = '*'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())

    def operacion(self):
        return self.op1 * self.op2

    def ver_pasos(self):
        self.pasos = pasos_binomios_forma1_latex(self.op1, self.op2)

class BinomiosFormaII(DosOperandos):
    def __init__(self, op1, op2):
        super(BinomiosFormaII, self).__init__(op1, op2)
        self.oper = '*'
        self.enunciado = latex(self.op1.as_expr()) + ' ' + self.oper + ' ' + latex(self.op2.as_expr())

    def operacion(self):
        return self.op1 * self.op2

    def ver_pasos(self):
        self.pasos = pasos_binomios_forma2_latex(self.op1, self.op2)

class TrinomioAlCuadrado(UnOperando):
    def __init__(self, op1):
        super(TrinomioAlCuadrado, self).__init__(op1)
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return self.op1**2

    def ver_pasos(self):
        self.pasos = pasos_trinomio_al_cuadrado_latex(self.op1)



from random import randint, shuffle
import json
from sympy.abc import x, y
from algebreb.listas.lista import Lista
from algebreb.expresiones.fracciones_algebraicas import (fa_aleatoria, 
                                                         fa_propia_aleatoria, 
                                                         fa_impropia_aleatoria)
from algebreb.ejercicios.operaciones_fracciones import (SumaFracAlg, 
                                                        RestaFracAlg, 
                                                        MultFracAlg, 
                                                        DivFracAlg, 
                                                        SimpFracAlg)

class ListaSumaFraccionesAlgebraicas(Lista):
    def __init__(self, caracteristicas):
        super(ListaSumaFraccionesAlgebraicas, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes sumas de fracciones algebraicas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        raices = True
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            f1 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f2 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            suma = SumaFracAlg(f1, f2)
            suma.ver_pasos()

            f3 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f4 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            suma2 = SumaFracAlg(f3, f4)

            f5 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f6 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            suma3 = SumaFracAlg(f5, f6)

            f7 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f8 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            suma4 = SumaFracAlg(f7, f8)

            suma.respuestas.append(suma.res)
            suma.respuestas.append(suma2.res)
            suma.respuestas.append(suma3.res)
            suma.respuestas.append(suma4.res)

            shuffle(suma.respuestas)

            lista.append(suma)

        return lista

class ListaRestaFraccionesAlgebraicas(Lista):
    def __init__(self, caracteristicas):
        super(ListaRestaFraccionesAlgebraicas, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes restas de fracciones algebraicas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        raices = True
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            f1 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f2 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            resta = RestaFracAlg(f1, f2)
            resta.ver_pasos()

            f3 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f4 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            resta2 = RestaFracAlg(f3, f4)

            f5 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f6 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            resta3 = RestaFracAlg(f5, f6)

            f7 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f8 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            resta4 = RestaFracAlg(f7, f8)

            resta.respuestas.append(resta.res)
            resta.respuestas.append(resta2.res)
            resta.respuestas.append(resta3.res)
            resta.respuestas.append(resta4.res)

            shuffle(resta.respuestas)
            
            lista.append(resta)

        return lista

class ListaMultFraccionesAlgebraicas(Lista):
    def __init__(self, caracteristicas):
        super(ListaMultFraccionesAlgebraicas, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes multiplicaciones de fracciones algebraicas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        raices = True
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            f1 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f2 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            mult = MultFracAlg(f1, f2)
            mult.ver_pasos()

            f3 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f4 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            mult2 = MultFracAlg(f3, f4)

            f5 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f6 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            mult3 = MultFracAlg(f5, f6)

            f7 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f8 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            mult4 = MultFracAlg(f7, f8)

            mult.respuestas.append(mult.res)
            mult.respuestas.append(mult2.res)
            mult.respuestas.append(mult3.res)
            mult.respuestas.append(mult4.res)

            shuffle(mult.respuestas)
                
            lista.append(mult)

        return lista

class ListaDivFraccionesAlgebraicas(Lista):
    def __init__(self, caracteristicas):
        super(ListaDivFraccionesAlgebraicas, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes divisiones de fracciones algebraicas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        raices = True
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            f1 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f2 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            div = DivFracAlg(f1, f2)
            div.ver_pasos()

            f3 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f4 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            div2 = DivFracAlg(f3, f4)

            f5 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f6 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            div3 = DivFracAlg(f5, f6)

            f7 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            f8 = fa_propia_aleatoria(grado2, variables, dominio, cmin, cmax, fraccion, raices)
            div4 = DivFracAlg(f7, f8)

            div.respuestas.append(div.res)
            div.respuestas.append(div2.res)
            div.respuestas.append(div3.res)
            div.respuestas.append(div4.res)

            shuffle(div.respuestas)

            lista.append(div)

        return lista

class ListaSimpFraccionesAlgebraicas(Lista):
    def __init__(self, caracteristicas):
        super(ListaSimpFraccionesAlgebraicas, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes simplificaciones de fracciones algebraicas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        raices = True
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)

            f1 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            sim = SimpFracAlg(f1)
            sim.ver_pasos()

            f2 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            sim2 = SimpFracAlg(f2)

            f3 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            sim3 = SimpFracAlg(f3)

            f4 = fa_propia_aleatoria(grado1, variables, dominio, cmin, cmax, fraccion, raices)
            sim4 = SimpFracAlg(f4)

            sim.respuestas.append(sim.res)
            sim.respuestas.append(sim2.res)
            sim.respuestas.append(sim3.res)
            sim.respuestas.append(sim4.res)

            shuffle(sim.respuestas)
                
            lista.append(sim)

        return lista


from sympy import *
from algebreb.ejercicios.tipos_ejercicios import UnOperando, DosOperandos
from algebreb.pasos.polinomios import pasos_mult_polinomios, pasos_suma_polinomios, pasos_resta_polinomios

class SumaPolinomios(DosOperandos):
    def __init__(self, p1, p2):
        super(SumaPolinomios, self).__init__(p1, p2)
        self.oper = '+'
        self.enunciado = '(' + latex(self.op1.as_expr()) + ') ' + self.oper + ' (' + latex(self.op2.as_expr()) + ')'

    def operacion(self):
        return self.op1 + self.op2

    def ver_pasos(self):
        self.pasos = pasos_suma_polinomios(self.op1, self.op2)

class RestaPolinomios(DosOperandos):
    def __init__(self, p1, p2):
        super(RestaPolinomios, self).__init__(p1, p2)
        self.oper = '-'
        self.enunciado = '(' + latex(self.op1.as_expr()) + ') ' + self.oper + ' (' + latex(self.op2.as_expr()) + ')'

    def operacion(self):
        return self.op1 - self.op2

    def ver_pasos(self):
        self.pasos = pasos_resta_polinomios(self.op1, self.op2)

class MultPolinomios(DosOperandos):
    def __init__(self, p1, p2):
        super(MultPolinomios, self).__init__(p1, p2)
        self.oper = '*'
        self.enunciado = '(' + latex(self.op1.as_expr()) + ') ' + self.oper + ' (' + latex(self.op2.as_expr()) + ')'

    def operacion(self):
        return self.op1 * self.op2

    def ver_pasos(self):
        self.pasos = pasos_mult_polinomios(self.op1, self.op2)

class DivPolinomios(DosOperandos):
    def __init__(self, p1, p2):
        super(DivPolinomios, self).__init__(p1, p2)
        self.oper = '/'
        self.enunciado = '(' + latex(self.op1.as_expr()) + ') ' + self.oper + ' (' + latex(self.op2.as_expr()) + ')'
        self.cociente, self.residuo = self.operacion()

    def operacion(self):
        return self.op1.div(self.op2)

    def as_str(self):
        dict = {
            'operacion': '/',
            'operando1': str(self.op1.as_expr()),
            'operando2': str(self.op2.as_expr()),
            'solucion': str(self.cociente.as_expr()),
            'residuo': str(self.residuo.as_expr()),
            'respuestas': [str(r.as_expr()) for r in self.respuestas]
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': '/',
            'operando1': latex(self.op1.as_expr()),
            'operando2': latex(self.op2.as_expr()),
            'solucion': latex(self.cociente.as_expr()),
            'residuo': latex(self.residuo.as_expr()),
            'respuestas': [latex(r.as_expr()) for r in self.respuestas],
            'enunciado': self.enunciado
        }

        return dict
    
class GradoPolinomio(UnOperando):
    def __init__(self, op1):
        super(GradoPolinomio, self).__init__(op1)
        self.enunciado = latex(self.op1.as_expr())
    
    def operacion(self):
        grado = self.op1.total_degree()

        return grado

    def as_str(self):
        dict = {
            'operacion': self.oper,
            'operando1': str(self.op1.as_expr()),
            'solucion': str(self.res),
            'respuestas': self.respuestas
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper, 
            'operando1': latex(self.op1.as_expr()),
            'solucion': self.res,
            'respuestas': self.respuestas,
            'enunciado' : self.enunciado
        }

        return dict

class TermPolinomio(UnOperando):
    def __init__(self, op1):
        super(TermPolinomio, self).__init__(op1)
        self.enunciado = latex(self.op1.as_expr())

    def operacion(self):
        num_terms = len(self.op1.as_dict())

        if num_terms == 1:
            return '\\textrm{Monomio}'
        elif num_terms == 2:
            return '\\textrm{Binomio}'
        elif num_terms == 3:
            return '\\textrm{Trinomio}'
        else:
            return '\\textrm{Polinomio}'

    def as_str(self):
        dict = {
            'operacion': self.oper,
            'operando1': str(self.op1.as_expr()),
            'solucion': self.res,
            'respuestas': self.respuestas
        }

        return dict

    def as_latex(self):
        dict = {
            'operacion': self.oper, 
            'operando1': latex(self.op1.as_expr()),
            'solucion': self.res,
            'respuestas': self.respuestas,
            'enunciado': self.enunciado
        }

        return dict


from fractions import Fraction
from algebreb.util.numeros import fraccion_aleatoria

frac1 = fraccion_aleatoria(-4, 3)
print(frac1)

excepcion = Fraction(1, 2)
frac2 = fraccion_aleatoria(1, 2, excepcion)
print(frac2)


from random import choice

from sympy.polys.monomials import itermonomials
from sympy.polys.orderings import monomial_key
from sympy.abc import x, y, z

def generar_terminos(vars, grado):
    return sorted(itermonomials(vars, grado), key=monomial_key('igrlex', vars))

def generar_terminos_grado(vars, grado):
    return sorted(itermonomials(vars, grado, grado), key=monomial_key('igrlex', vars))

def generar_terminos_restantes(vars, grado):
    terms1 = set(generar_terminos(vars, grado))
    terms2 = set(generar_terminos_grado(vars, grado))

    terms = list(terms1 - terms2)

    return sorted(terms, key=monomial_key('igrlex', vars))

def terminos_polinomio_homogeneo(vars, grado):
    terms1 = set(itermonomials(vars, grado))
    terms2 = set(itermonomials(vars, grado - 1))

    terms = list(terms1 - terms2)

    return sorted(terms, key=monomial_key('igrlex', vars))

def termino_aleatorio(vars, grado):
    termino = []

    termino.append(choice(terminos_polinomio_homogeneo(vars, grado)))

    return termino


from random import choice

# from Sympy
from sympy import Poly, Add

from sympy.abc import (a, b, c, d, e, f, g, h, i, 
                       j, k, l, m, n, o, p, q, r, 
                       s, t, u, v, w, x, y, z)

# from Algebreb
from algebreb.util.numeros_proxy import (numeros_no_nulos,
                          numeros)

from algebreb.util.terminos import (generar_terminos,
                      generar_terminos_grado,
                      generar_terminos_restantes, 
                      terminos_polinomio_homogeneo,
                      termino_aleatorio)

from algebreb.util.util import suma_azar, vieta, mcm_fracciones

class Polinomio(Poly):
    pass

def polinomio_coeficientes_aleatorios(grado, variables, dominio, coef_min, coef_max, fraccion=False, completo=True):
    terminos = []
    coefs = []

    terminos_grado = generar_terminos_grado(variables, grado)
    num_terminos_grado = len(terminos_grado)

    coeficientes_grado = numeros_no_nulos(dominio, num_terminos_grado, coef_min, coef_max, fraccion)

    terminos_restantes = generar_terminos_restantes(variables, grado)
    num_terminos_restantes = len(terminos_restantes)

    if completo:
        coeficientes_restantes = numeros_no_nulos(dominio, num_terminos_restantes, coef_min, coef_max, fraccion)
    else:
        nulos, no_nulos = suma_azar(num_terminos_restantes)
        coeficientes_restantes = numeros(dominio, nulos, no_nulos, coef_min, coef_max, fraccion)

    terminos = terminos_grado + terminos_restantes
    coefs = coeficientes_grado + coeficientes_restantes

    return generar_polinomio(coefs, terminos, variables)

def polinomio_raices_aleatorias(num_raices, variable, dominio, min, max, fraccion=False, nulas=False, coefs_frac=False):
    if nulas == False:
        raices = numeros_no_nulos(dominio, num_raices, min, max, fraccion)
    else:
        nulos, no_nulos = suma_azar(num_raices)
        raices = numeros(dominio, nulos, no_nulos, min, max, fraccion)

    coeficientes = vieta(raices, num_raices)

    terminos = generar_terminos(variable, num_raices)

    polinomio = generar_polinomio(coeficientes, terminos, variable)

    if dominio == 'QQ' and fraccion == True and coefs_frac==True:
        mcm = mcm_fracciones(coeficientes)

        #if coefs_frac==False:
        return polinomio * mcm
        #else:
            #polinomio

    return polinomio
    

def polinomio_homogeneo_aleatorio(grado, variables, dominio, coef_min, coef_max, fraccion=False, completo=True):
    terminos = []
    coeficientes = []

    terminos = terminos_polinomio_homogeneo(variables, grado)
    num_terminos = len(terminos)

    if completo:
        coeficientes = numeros_no_nulos(dominio, num_terminos, coef_min, coef_max, fraccion)
    else:
        nulos, no_nulos = suma_azar(num_terminos)
        coeficientes = numeros(dominio, nulos, no_nulos, coef_min, coef_max, fraccion)

    return generar_polinomio(coeficientes, terminos, variables)

def polinomio_semejante(polinomio):
    poli = polinomio

    coefs = poli.coeffs()
    terms = poli.monoms()

    min_coef = min(coefs)
    max_coef = max(coefs)

    return

def trinomio_aleatorio(grado, variables, dominio, coef_min, coef_max, fraccion=False):
    termino1 = choice(generar_terminos_grado(variables, grado))

    terminos_restantes = generar_terminos_restantes(variables, grado)

    termino2 = choice(terminos_restantes)

    terminos_restantes.remove(termino2)

    termino3 = choice(terminos_restantes)

    terminos = [termino1, termino2, termino3]
    coeficientes = numeros_no_nulos(dominio, 3, coef_min, coef_max, fraccion)

    return generar_polinomio(coeficientes, terminos, variables)

def binomio_aleatorio(grado, variables, dominio, coef_min, coef_max, fraccion=False):
    termino1 = choice(generar_terminos_grado(variables, grado))
    termino2 = choice(generar_terminos_restantes(variables, grado))

    terminos = [termino1, termino2]
    coeficientes = numeros_no_nulos(dominio, 2, coef_min, coef_max, fraccion)

    return generar_polinomio(coeficientes, terminos, variables)

def conjugar_binomio(binomio):
    binom = binomio

    terms_binom = binom.monoms()
    coefs_binom = binom.coeffs()

    coefs_binom[1] *= -1

    conjugado = Polinomio.from_dict({m: c for (m, c) in zip(terms_binom, coefs_binom)}, binom.gens)

    return conjugado

def monomio_aleatorio(grado, variables, dominio, coef_min, coef_max, fraccion=False):
    termino = termino_aleatorio(variables, grado)
    coeficiente = numeros_no_nulos(dominio, 1, coef_min, coef_max, fraccion)

    return generar_polinomio(coeficiente, termino, variables)

def generar_polinomio(coefs, terms, variables):
    p = Add(*[coef*term for coef, term in zip(coefs, terms)])

    return Polinomio(p, variables)


from random import randint, choice, shuffle, sample
from sympy import *
from sympy.abc import x, y
from algebreb.listas.lista import Lista
from algebreb.expresiones.polinomios import Polinomio, polinomio_raices_aleatorias
from algebreb.ejercicios.ecuaciones_univariables import EcuacionLineal, EcuacionCuadratica

class ListaEcuacionesGrado1(Lista):
    def __init__(self, caracteristicas):
        super(ListaEcuacionesGrado1, self).__init__(caracteristicas)
        print(caracteristicas)
        self.instrucciones = 'Resolver las siguientes ecuaciones lineales:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            ld = polinomio_raices_aleatorias(1, variables, dominio, cmin, cmax, fraccion)
            li = Polinomio(0, variables)
            ec = EcuacionLineal(ld, li)

            r1 = 0
            r2 = 0
            r3 = 0
            while(r1 == r2 or r1 == r3 or r2 == r3):
                r1 = randint(cmin, cmax+5)
                r2 = randint(cmin, cmax+5)
                r3 = randint(cmin, cmax+5)
                while(r1 == int(ec.res[0])):
                    r1 = randint(cmin, cmax+5)
                while(r2 == int(ec.res[0])):
                    r2 = randint(cmin, cmax+5)
                while(r3 == int(ec.res[0])):
                    r3 = randint(cmin, cmax+5)

            ec.respuestas.append(r1)
            ec.respuestas.append(r2)
            ec.respuestas.append(r3)
            ec.respuestas.append(int(ec.res[0]))

            lista.append(ec)

        return lista

class ListaEcuacionesGrado2(Lista):
    def __init__(self, caracteristicas):
        super(ListaEcuacionesGrado2, self).__init__(caracteristicas)
        print(caracteristicas)
        self.instrucciones = 'Resolver las siguientes ecuaciones cuadráticas:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            ld = polinomio_raices_aleatorias(2, variables, dominio, cmin, cmax, fraccion)
            li = Polinomio(0, variables)
            ec = EcuacionCuadratica(ld, li)

            res = sample(range(cmax*(-1)-10,cmax+10), 6)

            ec.respuestas.append('{},{}'.format(res[0], res[3]))
            ec.respuestas.append('{},{}'.format(res[1], res[4]))
            ec.respuestas.append('{},{}'.format(res[2], res[5]))
            ec.respuestas.append(','.join(str(r) for r in  ec.res))

            lista.append(ec)

        return lista


from random import randint, choice, shuffle
from sympy import *
from sympy.abc import x, y
from algebreb.listas.lista import Lista
from algebreb.expresiones.polinomios import (polinomio_coeficientes_aleatorios,
                                             polinomio_raices_aleatorias)

from algebreb.ejercicios.operaciones_polinomio import (SumaPolinomios, 
                                                       RestaPolinomios, 
                                                       MultPolinomios,
                                                       DivPolinomios,
                                                       GradoPolinomio,
                                                       TermPolinomio) 

import json

class ListaSumaPolinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaSumaPolinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes sumas de polinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            p1 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p2 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            suma = SumaPolinomios(p1, p2)
            suma.ver_pasos()
            
            p3 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p4 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            suma2 = SumaPolinomios(p3, p4)

            p5 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p6 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            suma3 = SumaPolinomios(p5, p6)

            p7 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p8 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            suma4 = SumaPolinomios(p7, p8)

            suma.respuestas.append(suma.res)
            suma.respuestas.append(suma2.res)
            suma.respuestas.append(suma3.res)
            suma.respuestas.append(suma4.res)

            shuffle(suma.respuestas)

            lista.append(suma)

        return lista

class ListaRestaPolinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaRestaPolinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes restas de polinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            p1 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p2 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            resta = RestaPolinomios(p1, p2)
            resta.ver_pasos()

            p3 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p4 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            resta2 = RestaPolinomios(p3, p4)

            p5 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p6 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            resta3 = RestaPolinomios(p5, p6)

            p7 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p8 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            resta4 = RestaPolinomios(p7, p8)

            resta.respuestas.append(resta.res)
            resta.respuestas.append(resta2.res)
            resta.respuestas.append(resta3.res)
            resta.respuestas.append(resta4.res)

            shuffle(resta.respuestas)


            lista.append(resta)

        return lista

class ListaMultPolinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaMultPolinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes multiplicaciones de polinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            p1 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p2 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            mult = MultPolinomios(p1, p2)
            mult.ver_pasos()

            p3 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p4 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            mult2 = MultPolinomios(p3, p4)

            p5 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p6 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            mult3 = MultPolinomios(p5, p6)

            p7 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)
            p8 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            mult4 = MultPolinomios(p7, p8)

            mult.respuestas.append(mult.res)
            mult.respuestas.append(mult2.res)
            mult.respuestas.append(mult3.res)
            mult.respuestas.append(mult4.res)

            shuffle(mult.respuestas)

            lista.append(mult)

        return lista

class ListaDivPolinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaDivPolinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar las siguientes divisiones de polinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        exacta = self.caracteristicas['exacta']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            if exacta:
                grado1 = randint(gmin, gmax)
                grado2 = randint(gmin, gmax)

                p1 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p2 = polinomio_raices_aleatorias(grado2, variables, dominio, cmin, cmax, fraccion, completo)
                producto = p1 * p2
                divisor = choice([p1, p2])

                div = DivPolinomios(producto, divisor)

                p3 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p4 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                producto2 = p3 * p4
                divisor2 = choice([p3, p4])
                div2 = DivPolinomios(producto2, divisor2)

                p5 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p6 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                producto3 = p5 * p6
                divisor3 = choice([p5, p6])
                div3 = DivPolinomios(producto3, divisor3)

                p7 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p8 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                producto4 = p7 * p8
                divisor4 = choice([p7, p8])
                div4 = DivPolinomios(producto4, divisor4)

                div.respuestas.append(div.cociente)
                div.respuestas.append(div2.cociente)
                div.respuestas.append(div3.cociente)
                div.respuestas.append(div4.cociente)

                shuffle(div.respuestas)

                lista.append(div)
            else:
                grado1 = randint(gmin, gmax)
                grado2 = randint(gmin, gmax)

                p1 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p2 = polinomio_raices_aleatorias(grado2, variables, dominio, cmin, cmax, fraccion, completo)
                div = DivPolinomios(p1, p2)

                p3 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p4 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                div2 = DivPolinomios(p3, p4)

                p5 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p6 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                div3 = DivPolinomios(p5, p6)

                p7 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                p8 = polinomio_raices_aleatorias(grado1, variables, dominio, cmin, cmax, fraccion, completo)
                div4 = DivPolinomios(p7, p8)

                div.respuestas.append(div.cociente)
                div.respuestas.append(div2.cociente)
                div.respuestas.append(div3.cociente)
                div.respuestas.append(div4.cociente)

                shuffle(div.respuestas)

                lista.append(div)

        return lista

class ListaGradoPolinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaGradoPolinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Mencionar el grado de los siguientes polinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)

            p1 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)

            grado = GradoPolinomio(p1)

            r1 = 0
            r2 = 0
            r3 = 0
            while(r1 == r2 or r1 == r3 or r2 == r3):
                r1 = randint(gmin, gmax+5)
                r2 = randint(gmin, gmax+5)
                r3 = randint(gmin, gmax+5)
                while(r1 == grado.res):
                    r1 = randint(gmin, gmax+5)
                while(r2 == grado.res):
                    r2 = randint(gmin, gmax+5)
                while(r3 == grado.res):
                    r3 = randint(gmin, gmax+5)
            
            grado.respuestas.append(r1)
            grado.respuestas.append(r2)
            grado.respuestas.append(r3)
            grado.respuestas.append(grado.res)

            shuffle(grado.respuestas)

            lista.append(grado)

        return lista

class ListaTermPolinomio(Lista):
    def __init__(self, caracteristicas):
        super(ListaTermPolinomio, self).__init__(caracteristicas)
        self.instrucciones = 'Mencionar el tipo de expresión (monomio, binomio, trinomio o polinomio):'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)

            p1 = polinomio_coeficientes_aleatorios(grado1, variables, dominio, cmin, cmax, fraccion, completo)

            term = TermPolinomio(p1)
            
            term.respuestas.append('\\textrm{Monomio}')
            term.respuestas.append('\\textrm{Binomio}')
            term.respuestas.append('\\textrm{Trinomio}')
            term.respuestas.append('\\textrm{Polinomio}')

            shuffle(term.respuestas)

            lista.append(term)

        return lista


import json
from random import randint, shuffle
from sympy import *
from sympy.abc import x, y
from algebreb.listas.lista import Lista
from algebreb.expresiones.polinomios import (monomio_aleatorio, 
                                             binomio_aleatorio, 
                                             trinomio_aleatorio, 
                                             polinomio_coeficientes_aleatorios,
                                             polinomio_raices_aleatorias,
                                             conjugar_binomio)

from algebreb.ejercicios.factorizacion import (CuboPerfectoBinomios, 
                                              DiferenciaCuadrados, 
                                              FactorComun, 
                                              TrinomioCuadradoPerfecto, 
                                              TrinomioFormaI, 
                                              TrinomioFormaII)

class ListaFactorComun(Lista):
    def __init__(self, caracteristicas):
        super(ListaFactorComun, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar las siguientes expresiones:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = 'ZZ'
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        completo = self.caracteristicas['completo']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            grado2 = randint(gmin, gmax)

            monomio = monomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            polinomio = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            producto = monomio*polinomio
            fc = FactorComun(producto)
            fc.monomio = monomio
            fc.polinomio = polinomio
            fc.producto = producto
            fc.ver_pasos()

            monomio2 = monomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            polinomio2 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            producto2 = monomio2 * polinomio2
            fc2 = FactorComun(producto2)

            monomio3 = monomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            polinomio3 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            producto3 = monomio3 * polinomio3
            fc3 = FactorComun(producto3)

            monomio4 = monomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            polinomio4 = polinomio_coeficientes_aleatorios(grado2, variables, dominio, cmin, cmax, fraccion, completo)
            producto4 = monomio4 * polinomio4
            fc4 = FactorComun(producto4)

            fc.respuestas.append(fc.res)
            fc.respuestas.append(fc2.res)
            fc.respuestas.append(fc3.res)
            fc.respuestas.append(fc4.res)

            shuffle(fc.respuestas)

            lista.append(fc)

        return lista

class ListaDiferenciaCuadrados(Lista):
    def __init__(self, caracteristicas):
        super(ListaDiferenciaCuadrados, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar las siguientes diferencias de cuadrados:'
      

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = False
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)

            ba = binomio_aleatorio(grado1, variables, 'ZZ', cmin, cmax, fraccion)
            bac = conjugar_binomio(ba)
            ba_expr = ba.as_expr()
            bac_expr = bac.as_expr()
            producto = ba * bac
            dc = DiferenciaCuadrados(producto)
            dc.res = UnevaluatedExpr(ba_expr*bac_expr)
            dc.bin1 = ba
            dc.bin2 = bac
            dc.producto = producto
            dc.ver_pasos()

            ba2 = binomio_aleatorio(grado1, variables, 'ZZ', cmin, cmax, fraccion)
            bac2 = conjugar_binomio(ba2)
            ba2_expr = ba2.as_expr()
            bac2_expr = bac2.as_expr()
            producto2 = ba2 * bac2
            dc2 = DiferenciaCuadrados(producto2)
            dc2.res = UnevaluatedExpr(ba2_expr * bac2_expr)

            ba3 = binomio_aleatorio(grado1, variables, 'ZZ', cmin, cmax, fraccion)
            bac3 = conjugar_binomio(ba3)
            ba3_expr = ba3.as_expr()
            bac3_expr = bac3.as_expr()
            producto3 = ba3 * bac3
            dc3 = DiferenciaCuadrados(producto3)
            dc3.res = UnevaluatedExpr(ba3_expr * bac3_expr)

            ba4 = binomio_aleatorio(grado1, variables, 'ZZ', cmin, cmax, fraccion)
            bac4 = conjugar_binomio(ba4)
            ba4_expr = ba4.as_expr()
            bac4_expr = bac4.as_expr()
            producto4 = ba4 * bac4
            dc4 = DiferenciaCuadrados(producto4)
            dc4.res = UnevaluatedExpr(ba4_expr * bac4_expr)

            dc.respuestas.append(dc.res)
            dc.respuestas.append(dc2.res)
            dc.respuestas.append(dc3.res)
            dc.respuestas.append(dc4.res)

            shuffle(dc.respuestas)

            lista.append(dc)

        return lista

class ListaTrinomioCuadradoPerfecto(Lista):
    def __init__(self, caracteristicas):
        super(ListaTrinomioCuadradoPerfecto, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar los siguientes trinomios cuadrados perfectos:'
        

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado = randint(gmin, gmax)

            ba = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod = ba**2
            ba_expr = ba.as_expr()
            tcp = TrinomioCuadradoPerfecto(prod)
            tcp.res = UnevaluatedExpr(ba_expr**2)
            tcp.binomio = ba
            tcp.producto = prod
            tcp.ver_pasos()

            ba2 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod2 = ba2**2
            ba2_expr = ba2.as_expr()
            tcp2 = TrinomioCuadradoPerfecto(prod2)
            tcp2.res = UnevaluatedExpr(ba2_expr**2)

            ba3 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod3 = ba3**2
            ba3_expr = ba3.as_expr()
            tcp3 = TrinomioCuadradoPerfecto(prod3)
            tcp3.res = UnevaluatedExpr(ba3_expr**2)

            ba4 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod4 = ba4**2
            ba4_expr = ba4.as_expr()
            tcp4 = TrinomioCuadradoPerfecto(prod4)
            tcp4.res = UnevaluatedExpr(ba4_expr**2)

            tcp.respuestas.append(tcp.res)
            tcp.respuestas.append(tcp2.res)
            tcp.respuestas.append(tcp3.res)
            tcp.respuestas.append(tcp4.res)

            shuffle(tcp.respuestas)

            lista.append(tcp)

        return lista

class ListaCuboPerfectoBinomios(Lista):
    def __init__(self, caracteristicas):
        super(ListaCuboPerfectoBinomios, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar los siguientes cubos perfectos de binomios:'
      

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado = randint(gmin, gmax)

            ba = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod = ba**3
            ba_expr = ba.as_expr()
            cpb = CuboPerfectoBinomios(prod)
            cpb.res = UnevaluatedExpr(ba_expr**3)
            cpb.binomio = ba
            cpb.producto = prod
            cpb.ver_pasos()

            ba2 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod2 = ba2**3
            ba2_expr = ba2.as_expr()
            cpb2 = CuboPerfectoBinomios(prod2)
            cpb2.res = UnevaluatedExpr(ba2_expr**3)

            ba3 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod3 = ba3**3
            ba3_expr = ba3.as_expr()
            cpb3 = CuboPerfectoBinomios(prod3)
            cpb3.res = UnevaluatedExpr(ba3_expr**3)

            ba4 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            prod4 = ba4**3
            ba4_expr = ba4.as_expr()
            cpb4 = CuboPerfectoBinomios(prod4)
            cpb4.res = UnevaluatedExpr(ba4_expr**3)

            cpb.respuestas.append(cpb.res)
            cpb.respuestas.append(cpb2.res)
            cpb.respuestas.append(cpb3.res)
            cpb.respuestas.append(cpb4.res)

            shuffle(cpb.respuestas)

            lista.append(cpb)

        return lista

class ListaTrinomioFormaI(Lista):
    def __init__(self, caracteristicas):
        super(ListaTrinomioFormaI, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar los siguientes trinomios:'
    

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        variables = self.caracteristicas['variables']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        lista = []

        for _ in range(cantidad):
            ba1 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba2 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba1_expr = ba1.as_expr()
            ba2_expr = ba2.as_expr()
            producto = ba1 * ba2
            tf1 = TrinomioFormaI(producto)
            tf1.res = UnevaluatedExpr(ba1_expr*ba2_expr)
            tf1.bin1 = ba1
            tf1.bin2 = ba2
            tf1.producto = producto
            tf1.ver_pasos()

            ba3 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba4 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba3_expr = ba3.as_expr()
            ba4_expr = ba4.as_expr()
            producto2 = ba3 * ba4
            tf2 = TrinomioFormaI(producto2)
            tf2.res = UnevaluatedExpr(ba3_expr*ba4_expr)

            ba5 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba6 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba5_expr = ba5.as_expr()
            ba6_expr = ba6.as_expr()
            producto3 = ba5 * ba6
            tf3 = TrinomioFormaI(producto3)
            tf3.res = UnevaluatedExpr(ba5_expr*ba6_expr)

            ba7 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba8 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba7_expr = ba7.as_expr()
            ba8_expr = ba8.as_expr()
            producto4 = ba7 * ba8
            tf4 = TrinomioFormaI(producto4)
            tf4.res = UnevaluatedExpr(ba7_expr*ba8_expr)

            tf1.respuestas.append(tf1.res)
            tf1.respuestas.append(tf2.res)
            tf1.respuestas.append(tf3.res)
            tf1.respuestas.append(tf4.res)

            shuffle(tf1.respuestas)

            lista.append(tf1)

        return lista 

class ListaTrinomioFormaII(Lista):
    def __init__(self, caracteristicas):
        super(ListaTrinomioFormaII, self).__init__(caracteristicas)
        self.instrucciones = 'Factorizar los siguientes trinomios:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        variables = self.caracteristicas['variables']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        lista = []

        for _ in range(cantidad):
            ba1 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=True)
            ba2 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=True)
            ba1_expr = ba1.as_expr()
            ba2_expr = ba2.as_expr()
            producto = ba1 * ba2
            tf2 = TrinomioFormaII(producto)
            tf2.res = UnevaluatedExpr(ba1_expr*ba2_expr)
            tf2.bin1 = ba1
            tf2.bin2 = ba2
            tf2.producto = producto
            tf2.ver_pasos()

            ba3 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba4 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba3_expr = ba3.as_expr()
            ba4_expr = ba4.as_expr()
            producto2 = ba3 * ba4
            tf3 = TrinomioFormaII(producto2)
            tf3.res = UnevaluatedExpr(ba3_expr*ba4_expr)

            ba5 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba6 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba5_expr = ba5.as_expr()
            ba6_expr = ba6.as_expr()
            producto3 = ba5 * ba6
            tf4 = TrinomioFormaII(producto3)
            tf4.res = UnevaluatedExpr(ba5_expr*ba6_expr)

            ba7 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba8 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba7_expr = ba7.as_expr()
            ba8_expr = ba8.as_expr()
            producto4 = ba7 * ba8
            tf5 = TrinomioFormaII(producto4)
            tf5.res = UnevaluatedExpr(ba7_expr*ba8_expr)

            tf2.respuestas.append(tf2.res)
            tf2.respuestas.append(tf3.res)
            tf2.respuestas.append(tf4.res)
            tf2.respuestas.append(tf5.res)

            shuffle(tf2.respuestas)

            lista.append(tf2)

        return lista


from random import shuffle
from algebreb.util.numeros import (nulos,
                     entero_aleatorio,
                     enteros_aleatorios,
                     decimal_aleatorio,
                     decimales_aleatorios,
                     fraccion_aleatoria,
                     fracciones_aleatorias)

def numeros(dominio, nulos, no_nulos, min, max, fraccion=False):
    nums_nulos = numeros_nulos(nulos)
    nums_no_nulos = numeros_no_nulos(dominio, no_nulos, min, max, fraccion)

    nums = nums_nulos + nums_no_nulos

    shuffle(nums)
    
    return nums

def numeros_nulos(cantidad):
    return nulos(cantidad)

def numeros_no_nulos(dominio, cantidad, min, max, fraccion=False):
    numeros = []

    if dominio == 'ZZ':
        numeros = enteros_aleatorios(min, max, cantidad, 0)
    elif dominio == 'QQ':
        if fraccion:
            numeros = fracciones_aleatorias(min, max, cantidad, 0)
        else:
            numeros = decimales_aleatorios(min, max, cantidad, 0)
    else:
        return numeros

    return numeros

def numero_no_nulo(dominio, min, max, fraccion=False):
    if dominio == 'ZZ':
        numero = entero_aleatorio(min, max, 0)
    elif dominio == 'QQ':
        if fraccion:
            numero = fraccion_aleatoria(min, max, 0)
        else:
            numero = decimal_aleatorio(min, max, 0)
    else:
        return

    return numero


from random import randint, shuffle
import json
from sympy.abc import x, y
from algebreb.listas.lista import Lista
from algebreb.expresiones.polinomios import (binomio_aleatorio, 
                                             trinomio_aleatorio,
                                             conjugar_binomio,
                                             polinomio_raices_aleatorias)
from algebreb.ejercicios.productos_notables import (BinomioAlCuadrado,
                                                    BinomioAlCubo, 
                                                    BinomiosConjugados, 
                                                    BinomiosFormaI, 
                                                    BinomiosFormaII,
                                                    TrinomioAlCuadrado)

class ListaBinomioAlCuadrado(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaBinomioAlCuadrado, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes binomios al cuadrado:'


    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado = randint(gmin, gmax)
            ba = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)

            bincuad = BinomioAlCuadrado(ba)
            bincuad.ver_pasos()

            r1 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincuad2 = BinomioAlCuadrado(r1)
            r2 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincuad3 = BinomioAlCuadrado(r2)
            r3 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincuad4 = BinomioAlCuadrado(r3)

            bincuad.respuestas.append(bincuad2.res)
            bincuad.respuestas.append(bincuad3.res)
            bincuad.respuestas.append(bincuad4.res)
            bincuad.respuestas.append(bincuad.res)

            shuffle(bincuad.respuestas)
            
            lista.append(bincuad)

        return lista

class ListaBinomioAlCubo(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaBinomioAlCubo, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes binomios al cubos:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado = randint(gmin, gmax)
            ba = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincub = BinomioAlCubo(ba)
            bincub.ver_pasos()

            r1 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincub2 = BinomioAlCubo(r1)
            r2 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincub3 = BinomioAlCubo(r2)
            r3 = binomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            bincub4 = BinomioAlCubo(r3)

            bincub.respuestas.append(bincub2.res)
            bincub.respuestas.append(bincub3.res)
            bincub.respuestas.append(bincub4.res)
            bincub.respuestas.append(bincub.res)

            shuffle(bincub.respuestas)

            lista.append(bincub)

        return lista 

class ListaBinomiosConjugados(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaBinomiosConjugados, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes binomios conjugados:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado1 = randint(gmin, gmax)
            ba = binomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            bac = conjugar_binomio(ba)
            binconj = BinomiosConjugados(ba, bac)
            binconj.ver_pasos()

            r1 = binomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            r1c = conjugar_binomio(r1)
            binconj2 = BinomiosConjugados(r1, r1c)

            r2 = binomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            r2c = conjugar_binomio(r2)
            binconj3 = BinomiosConjugados(r2, r2c)

            r3 = binomio_aleatorio(grado1, variables, dominio, cmin, cmax, fraccion)
            r3c = conjugar_binomio(r3)
            binconj4 = BinomiosConjugados(r3, r3c)

            binconj.respuestas.append(binconj2.res)
            binconj.respuestas.append(binconj3.res)
            binconj.respuestas.append(binconj4.res)
            binconj.respuestas.append(binconj.res)

            shuffle(binconj.respuestas)

            lista.append(binconj)

        return lista 

class ListaBinomiosFormaI(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaBinomiosFormaI, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes binomios con término común:'

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        variables = self.caracteristicas['variables']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        lista = []

        for _ in range(cantidad):
            ba1 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            ba2 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            binf1 = BinomiosFormaI(ba1, ba2)
            binf1.ver_pasos()

            r1 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            r11 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            binf12 = BinomiosFormaI(r1, r11)

            r2 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            r22 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            binf13 = BinomiosFormaI(r2, r22)

            r3 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            r33 = polinomio_raices_aleatorias(1, variables, 'ZZ', cmin, cmax)
            binf14 = BinomiosFormaI(r3, r33)

            binf1.respuestas.append(binf12.res)
            binf1.respuestas.append(binf13.res)
            binf1.respuestas.append(binf14.res)
            binf1.respuestas.append(binf1.res)

            shuffle(binf1.respuestas)

            lista.append(binf1)

        return lista 

class ListaBinomiosFormaII(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaBinomiosFormaII, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes productos de binomios:'
    

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        variables = self.caracteristicas['variables']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        lista = []

        for _ in range(cantidad):
            ba1 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            ba2 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            binf2 = BinomiosFormaII(ba1, ba2)
            binf2.ver_pasos()

            r1 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            r11 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            binf22 = BinomiosFormaII(r1, r11)

            r2 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            r22 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            binf23 = BinomiosFormaII(r2, r22)

            r3 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            r33 = polinomio_raices_aleatorias(1, variables, 'QQ', cmin, cmax, True, nulas=False, coefs_frac=False)
            binf24 = BinomiosFormaII(r3, r33)

            binf2.respuestas.append(binf22.res)
            binf2.respuestas.append(binf23.res)
            binf2.respuestas.append(binf24.res)
            binf2.respuestas.append(binf2.res)

            shuffle(binf2.respuestas)

            lista.append(binf2)

        return lista

class ListaTrinomioAlCuadrado(Lista):
    def __init__(self, caracteristicas) -> None:
        super(ListaTrinomioAlCuadrado, self).__init__(caracteristicas)
        self.instrucciones = 'Realizar los siguientes trinomios al cuadrado:'
      

    def lista_ejercicios(self):
        cantidad = self.caracteristicas['cantidad']
        gmin = self.caracteristicas['gmin']
        gmax = self.caracteristicas['gmax']
        variables = self.caracteristicas['variables']
        dominio = self.caracteristicas['dominio']
        cmin = self.caracteristicas['cmin']
        cmax = self.caracteristicas['cmax']
        fraccion = self.caracteristicas['fraccion']
        lista = []

        for _ in range(cantidad):
            grado = randint(gmin, gmax)
            ba = trinomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            trincuad = TrinomioAlCuadrado(ba)
            trincuad.ver_pasos()

            r1 = trinomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            trincuad2 = TrinomioAlCuadrado(r1)
            r2 = trinomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            trincuad3 = TrinomioAlCuadrado(r2)
            r3 = trinomio_aleatorio(grado, variables, dominio, cmin, cmax, fraccion)
            trincuad4 = TrinomioAlCuadrado(r3)

            trincuad.respuestas.append(trincuad2.res)
            trincuad.respuestas.append(trincuad3.res)
            trincuad.respuestas.append(trincuad4.res)
            trincuad.respuestas.append(trincuad.res)

            shuffle(trincuad.respuestas)

            lista.append(trincuad)

        return lista


from fractions import Fraction
from algebreb.util.numeros import fracciones_aleatorias

fracs1 = fracciones_aleatorias(1, 10, 20)
print(fracs1)

excep = Fraction(1, 2)
fracs2 = fracciones_aleatorias(1, 2, 5, excep)
print(fracs2)


from sympy.solvers import solve
from sympy.abc import x
from sympy import *
from algebreb.expresiones.polinomios import Polinomio
from algebreb.ejercicios.tipos_ejercicios import DosOperandos

class SistemaEcuaciones(DosOperandos):
    def __init__(self, lado_derecho, lado_izquierdo) -> None:
        super(SistemaEcuaciones, self).__init__(lado_derecho, lado_izquierdo)
        self.oper = '='
        self.metodo = ''

    def operacion(self):
        solucion = solve((eq1, eq2), (x, y, z))

        return solucion


from algebreb.util.numeros import enteros_aleatorios

enteros1 = enteros_aleatorios(-100, 100, 10)
print(enteros1)

enteros2 = enteros_aleatorios(-7, 9, 10)
print(enteros2)

enteros3 = enteros_aleatorios(-1, 1, 10, 0)
print(enteros3)


from sympy import *
from sympy.abc import a, b, c, x, y, z
from sympy import UnevaluatedExpr
from algebreb.expresiones.polinomios import Polinomio

def textMath(text):
    inicio = '\\textrm{'
    fin = '}'
    return inicio + text + fin

def pasos_suma_polinomios(p1, p2):
    pasos = []

    p3 = p1 + p2

    pasos.append(textMath('Se encuentran los terminos semejantes del polinomio: ') 
        + latex(p1.as_expr()) + textMath(' con el polinomio ') + latex(p2.as_expr()) )
    # pasos.append(textMath('Los cuales son: '))
    pasos.append(textMath('Finalmente se tiene: ') + latex(p3.as_expr()))

    return pasos

def pasos_resta_polinomios(p1, p2):
    pasos = []

    p3 = p1 - p2

    pasos.append(textMath('Se encuentran los terminos semejantes del polinomio: ') 
        + latex(p1.as_expr()) + textMath(' con el polinomio ') + latex(p2.as_expr()) )
    # pasos.append(textMath('Los cuales son: '))
    pasos.append(textMath('Finalmente se tiene: ') + latex(p3.as_expr()))

    return pasos

def pasos_mult_polinomios(p1, p2):
    pasos = []
    sumandos = []

    monomios_p1 = list(Add.make_args(p1.as_expr()))
    paso = ''

    poli2_expr = p2.as_expr() 

    for monomio in monomios_p1:
        producto = monomio * poli2_expr
        # paso = 'Se multiplica el termino {} por {}, obteniendo {}'.format(latex(monomio), latex(poli2_expr), latex(producto))
        pasos.append( textMath('Se multiplica el termino ') 
            + latex(monomio) + textMath(' por ') + latex(poli2_expr) 
            + textMath(', obteniendo ') + latex(producto))
        sumandos.append(producto)
    
    str_sumandos = ' + '.join([latex(suma) for suma in sumandos])

    #pasos.append('Se suman los productos obtenidos: {}'.format(str_sumandos))
    pasos.append(textMath('Se suman los productos obtenidos: ') + str_sumandos)
    #pasos.append('Finalmente se obtiene: {}'.format(latex((p1*p2).as_expr())))
    pasos.append(textMath('Finalmente se obtiene: ') + latex((p1*p2).as_expr()))
    
    return pasos


poli1 = Polinomio(3*x**2+x+6, x)
poli2 = Polinomio(x**2+5*x+6, x)
multi = pasos_suma_polinomios(poli1, poli2)
print("PASOS: ", multi)
#print(poli1*poli2)


from sympy import *
from sympy.abc import x, y
from algebreb.ejercicios.tipos_ejercicios import UnOperando
from algebreb.expresiones.polinomios import Polinomio, monomio_aleatorio, polinomio_coeficientes_aleatorios
from algebreb.pasos.factorizacion import pasos_cubo_perfecto_binomios_latex, pasos_diferencia_cuadrados_latex, pasos_factor_comun_latex, pasos_trinomio_cuadrado_perfecto_latex, pasos_trinomio_forma1_latex, pasos_trinomio_forma2_latex

class FactorComun(UnOperando):
    def __init__(self, op1):
        super(FactorComun, self).__init__(op1)
        self.monomio = None
        self.polinomio = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_factor_comun_latex(self.monomio, self.polinomio, self.producto)

class DiferenciaCuadrados(UnOperando):
    def __init__(self, op1):
        super(DiferenciaCuadrados, self).__init__(op1)
        self.bin1 = None
        self.bin2 = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_diferencia_cuadrados_latex(self.bin1, self.bin2, self.producto)

class TrinomioCuadradoPerfecto(UnOperando):
    def __init__(self, op1):
        super(TrinomioCuadradoPerfecto, self).__init__(op1)
        self.binomio = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_trinomio_cuadrado_perfecto_latex(self.binomio, self.producto)

class CuboPerfectoBinomios(UnOperando):
    def __init__(self, op1):
        super(CuboPerfectoBinomios, self).__init__(op1)
        self.binomio = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_cubo_perfecto_binomios_latex(self.binomio, self.producto)

class TrinomioFormaI(UnOperando):
    def __init__(self, op1):
        super(TrinomioFormaI, self).__init__(op1)
        self.bin1 = None
        self.bin2 = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_trinomio_forma1_latex(self.bin1, self.bin2, self.producto)

class TrinomioFormaII(UnOperando):
    def __init__(self, op1):
        super(TrinomioFormaII, self).__init__(op1)
        self.bin1 = None
        self.bin2 = None
        self.producto = None
        self.oper = ''
        self.enunciado = self.oper + ' ' + latex(self.op1.as_expr())

    def operacion(self):
        return factor(self.op1.as_expr())

    def ver_pasos(self):
        self.pasos = pasos_trinomio_forma2_latex(self.bin1, self.bin2, self.producto)


from algebreb.ejercicios.operaciones_polinomio import (SumaPolinomios, RestaPolinomios,
                                                       MultPolinomios, DivPolinomios,
                                                       GradoPolinomio, TermPolinomio)
from algebreb.ejercicios.operaciones_fracciones import (SumaFracAlg, RestaFracAlg,
                                                        MultFracAlg, DivFracAlg,
                                                        SimpFracAlg)
from algebreb.ejercicios.productos_notables import (BinomioAlCuadrado, BinomioAlCubo,
                                                    BinomiosConjugados, BinomiosFormaI,
                                                    BinomiosFormaII, TrinomioAlCuadrado)
from algebreb.ejercicios.factorizacion import (FactorComun, DiferenciaCuadrados,
                                               TrinomioCuadradoPerfecto, CuboPerfectoBinomios,
                                               TrinomioFormaI, TrinomioFormaII)

def ejer_poli_factory(tipo, op1, op2=None):
    if tipo == '+':
        return SumaPolinomios(op1, op2)
    elif tipo == '-':
        return RestaPolinomios(op1, op2)
    elif tipo == '*':
        return MultPolinomios(op1, op2)
    elif tipo == '/':
        return DivPolinomios(op1, op2)
    elif tipo == 'GP':
        return GradoPolinomio(op1)
    elif tipo == 'TP':
        return TermPolinomio(op1)
    else:
        return None

def ejer_fa_factory(tipo, op1, op2=None):
    if tipo == '+':
        return

def ejer_prod_not_factory(tipo, op):
    if tipo == 'BA2':
        return BinomioAlCuadrado(op)
    elif tipo == 'BA3':
        return BinomioAlCubo(op)
    elif tipo == 'BC':
        return BinomiosConjugados(op)
    elif tipo == 'BF1':
        return BinomiosFormaI(op)
    elif tipo == 'BF2':
        return BinomiosFormaII(op)
    elif tipo == 'TA2':
        return TrinomioAlCuadrado(op)

def ejer_factor_factory(tipo, op):
    return